@@ -1,6 +1,6 @@
 #   Our beloved Emacs will give us -*- perl -*- mode :-)
 #
-#   $Id: mysql.pm,v 1.1.1.1 1997/08/27 10:32:15 joe Exp $
+#   $Id: mSQL.pm,v 1.1803 1997/08/28 23:38:57 joe Exp $
 #
 #   Copyright (c) 1994,1995,1996,1997 Alligator Descartes, Tim Bunce
 #
@@ -83,6 +83,7 @@ sub connect {
 	if (ref($attrhash)  &&  ref($attrhash) eq 'HASH') {
 	    if (defined($hostname = $attrhash->{'hostname'})) {
 		$dsn = "$dsn:hostname=$hostname";
+	    
 	    }
 	}
     }
@@ -119,7 +120,7 @@ sub data_sources {
     my(@dsn) = $self->func('', '_ListDBs');
     my($i);
     for ($i = 0;  $i < @dsn;  $i++) {
-	$dsn[$i] = "DBI:mSQL:$dsn[$i]";
+	$dsn[$i] = "DBI:mysql:$dsn[$i]";
     }
     @dsn;
 }
@@ -210,11 +211,20 @@ __END__
 
 =head1 NAME
 
-DBD::mysql - mSQL-1.I<x> / 2.I<x> driver for the Perl5 Database Interface (DBI)
+DBD::mSQL / DBD::mysql - mSQL and mysql drivers for the Perl5 Database
+Interface (DBI)
 
 =head1 SYNOPSIS
 
-    $dbh = DBI->connect( "$database:$hostname:$port" );
+    use DBI;
+
+    $dbh = DBI->connect("DBI:mSQL:$database:$hostname:$port",
+			undef, undef);
+
+        or
+
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname:$port",
+			$user, $password);
 
     @databases = $drh->func( $hostname, '_ListDBs' );
     @tables = $dbh->func( '_ListTables' );
@@ -228,48 +238,84 @@ DBD::mysql - mSQL-1.I<x> / 2.I<x> driver for the Perl5 Database Interface (DBI)
 
 =head1 DESCRIPTION
 
-<DBD::mysql> is the Perl5 Database Interface driver for mSQL 1.I<x> and
-2.I<x> databases.
+<DBD::mysql> and <DBD::mSQL> are the Perl5 Database Interface drivers for
+the mysql, mSQL 1.I<x> and mSQL 2.I<x> databases. The drivers are part
+of the I<mysql-modules> and I<Msql-modules> packages, respectively.
 
 =head2 Compatibility Alert
 
-As of version 0.70 DBD::mysql has a new maintainer
+As of version 0.70 DBD::mSQL has a new maintainer. Even more, the sources
+have been completely rewritten in August 1997, so it seemed apropriate
+to bump the version number: Incompatibilities are more than likely.
 
 
-=head2 DBD::mysql Class Methods
+=head2 Class Methods
 
 =over 4
 
 =item B<connect>
 
-    $dbh = DBI->connect( "$database" );
-    $dbh = DBI->connect( "$database:$hostname" );
-    $dbh = DBI->connect( "$database:$hostname:$port" );
+    use DBI;
+
+    $dbh = DBI->connect("DBI:mSQL:$database", undef, undef);
+    $dbh = DBI->connect("DBI:mSQL:$database:$hostname", undef, undef);
+    $dbh = DBI->connect("DBI:mSQL:$database:$hostname:$port",
+			undef, undef);
+
+        or
+
+    use DBI;
+
+    $dbh = DBI->connect("DBI:mysql:$database", $user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname",
+			$user, $password);
+    $dbh = DBI->connect("DBI:mysql:$database:$hostname:$port",
+			$user, $password);
 
 A C<database> must always be specified.
 
 The hostname, if not specified or specified as '', will default to an
-mSQL daemon running on the local machine on the default port for the
-UNIX socket.
+mysql or mSQL daemon running on the local machine on the default port
+for the UNIX socket.
 
-Should the mSQL daemon be running on a non-standard port number, you
-may explicitly state the port number to connect to in the C<hostname>
+Should the mysql or mSQL daemon be running on a non-standard port number,
+you may explicitly state the port number to connect to in the C<hostname>
 argument, by concatenating the I<hostname> and I<port number> together
 separated by a colon ( C<:> ) character.
 
 =back
 
-=head2 DBD::mysql Private MetaData Methods
+=head2 Private MetaData Methods
 
 =over 4
 
+=item B<ListDBs>
+
+    @dbs = $dbh->func("$hostname:$port", '_ListDBs');
+
+Returns a list of all databases managed by the mysql daemon or
+mSQL daemon running on C<$hostname>, port C<$port>. This method
+is rarely needed for databases running on C<localhost>: You should
+use the portable method
+
+    @dbs = DBI->data_sources("mysql");
+
+        or
+
+    @dbs = DBI->data_sources("mSQL");
+
+whenever possible. It is a design problem of this method, that there's
+no way of supplying a host name or port number to C<data_sources>, that's
+the only reason why we still support C<ListDBs>. :-(
+
+
 =item B<ListTables>
 
-    @tables = $dbh->func( '_ListTables' );
+    @tables = $dbh->func('_ListTables');
 
-Once connected to the desired database on the desired mSQL daemon with
-the C<DBI->connect()> method, we may extract a list of the tables that
-have been created within that database.
+Once connected to the desired database on the desired mysql or mSQL
+mSQL daemon with the C<DBI->connect()> method, we may extract a list
+of the tables that have been created within that database.
 
 C<ListTables> returns an array containing the names of all the tables
 present within the selected database. If no tables have been created,
@@ -280,6 +326,7 @@ an empty list is returned.
         print "Table: $table\n";
       }
 
+
 =item B<ListFields>
 
     $ref = $dbh->func( $table, '_ListFields' );
@@ -299,6 +346,21 @@ The valid keys within the hashtable that may be referenced are:
     LENGTH         The size of the field
     NUMFIELDS      The number of fields within the table
 
+Mysql also supports the following keys:
+
+    IS_KEY         Indicates whether the field is part of any key.
+    IS_BLOB        Indicates whether the given field is of blob type;
+                   don't put too much interpretation in this: A char
+                   field may contain binary data too, though size
+                   restricted.
+    IS_NUM         Indicates whether the given field contains numeric
+                   data. (The driver will return strings anyways,
+                   as the database does.)
+    MAXLENGTH      Perhaps the maximum length of all fields of the
+                   given column; as far as I know this isn't documented,
+                   so don't trust in that. (Monty?)
+    
+
 Since a reference is returned, it requires slightly more work to
 extract the pertinent information from it. Here's an example of how to
 do it:
@@ -317,6 +379,7 @@ do it:
         print "\tLength: $fieldLength[$i]\n";
       }
 
+
 =item B<ListSelectedFields>
 
     $ref = $sth->func( '_ListSelectedFields' );
@@ -327,16 +390,25 @@ current database, C<ListSelectedFields> lists the field information
 for the fields present in a B<SELECT> statement handle. This is
 primarily used for extracting meta-data about the current C<sth>.
 
-The usage of C<ListSelectedFields> is identical to C<ListFields>.
+The usage of C<ListSelectedFields> is identical to C<ListFields>, but
+you might use the following additional keys:
+
+    NUMFIELDS       The number of fields selected in one row; this
+                    should be valid even if the query didn't return
+                    any rows.
+    NUMROWS         The number of rows selected.
 
 =back
 
-=head2 DBD::mysql Database Manipulation
+
+=head2 Database Manipulation
 
 =over 4
 
 =item B<CreateDB>
 
+=item B<DropDB>
+
     $rc = $drh->func( $database, '_CreateDB' );
     $rc = $drh->func( $database, '_DropDB' );
 
@@ -355,6 +427,7 @@ These methods should be used at your own risk.
 
 =back
 
+
 =head1 BUGS
 
 The I<port> part of the first argument to the connect call is
@@ -362,42 +435,80 @@ implemented in an unsafe way. In fact it never did more than set
 the environment variable MSQL_TCP_PORT during the connect call. If
 another connect call uses another port and the handles are used
 simultaneously, they will interfere. In a future version this
-behaviour will change.
-
-The I<host> part of the first argument to the connect call is
-currently documented as defaulting to 'localhost'. If I read this
-right, it implicates that there are no provisions to connect to the
-UNIX socket. This is a major speed disadvantage for application that
-run on the server host. This will have to be revisited in the next
-release.
+behaviour will hoefully change, depending on David and Monty. :-)
 
 The func method call on a driver handle seems to be undocumented in
-the DBI manpage. DBD::mysql has func methods on driverhandles, database
+the DBI manpage. DBD::mSQL has func methods on driverhandles, database
 handles, and statement handles. What gives?
 
-Despite all these func methods, AFAIK it is currently not possible to
-connect to a different host and query the available databases. If
-true, this is a minor nit, but needs to be resolved somehow.
-
-I haven't yet found out how the constants CHAR_TYPE, INT_TYPE,
-etc. are accessed in DBD::mysql. Can anybody help me on the tracks
-here?
-
 Please speak up now (June 1997) if you encounter additional bugs. I'm
 still learning about the DBI API and can neither judge the quality of
 the code presented here nor the DBI compliancy. But I'm intending to
 resolve things quickly as I'd really like to get rid of the multitude
 of implementations ASAP.
 
+When running "make test", you will notice that some test scripts fail.
+This is due to bugs in the respective databases, not in the DBI drivers:
+
+=over 4
+
+=item Nulls
+
+mSQL seems to have problems with NULL's: The following fails with
+mSQL 2.0.1 running on a Linux 2.0.30 machine:
+
+    [joe@laptop Msql-modules-1.18]$ msql test
+    Welcome to the miniSQL monitor.  Type \h for help.
+    mSQL > CREATE TABLE foo (id INTEGER, name CHAR(6))\g
+    Query OK.  1 row(s) modified or retrieved.
+    mSQL > INSERT INTO foo VALUES (NULL, 'joe')\g
+    Query OK.  1 row(s) modified or retrieved.
+    mSQL > SELECT * FROM foo WHERE id = NULL\g
+    Query OK.  0 row(s) modified or retrieved.
+    +----------+------+
+    | id       | name |
+    +----------+------+
+    +----------+------+
+    mSQL > 
+
+=item Blanks
+
+mysql has problems with Blanks on the right side of string fields: They
+get chopped of. (Tested with mysql 3.20.25 on a Linux 2.0.30 machine.)
+
+    [joe@laptop Msql-modules-1.18]$ mysql test
+    Welcome to the mysql monitor.  Commands ends with ; or \g.
+    Type 'help' for help.
+    mysql> CREATE TABLE foo (id INTEGER, bar CHAR(8));
+    Query OK, 0 rows affected (0.10 sec)
+    mysql> INSERT INTO foo VALUES (1, ' a b c ');
+    Query OK, 1 rows affected (0.00 sec)
+    mysql> SELECT * FROM foo;
+    1 rows in set (0.19 sec)
+    +------+--------+
+    | id   | bar    |
+    +------+--------+
+    |    1 |  a b c |
+    +------+--------+
+    mysql> quit;
+    [joe@laptop Msql-modules-1.18]$ mysqldump test foo
+
+    [deleted]
+
+    INSERT INTO foo VALUES (1,' a b c');
+
+=back
+
 =head1 AUTHOR
 
-B<DBD::mysql> has been primarily written by Alligator Descartes
+B<DBD::mSQL> has been primarily written by Alligator Descartes
 <I<descarte@hermetica.com>>, who has been aided and abetted by Gary
 Shea, Andreas Koenig and Tim Bunce amongst others. Apologies if your
 name isn't listed, it probably is in the file called
 'Acknowledgments'. As of version 0.80 the maintainer is Andreas Knig.
 Version 2.00 is an almost complete rewrite by Jochen Wiedmann.
 
+
 =head1 COPYRIGHT
 
 This module is Copyright (c)1997 Jochen Wiedmann, with code portions
@@ -415,6 +526,7 @@ distribution rights, including incorporation into commercial products,
 such as books, magazine articles or CD-ROMs should be made to
 Alligator Descartes <I<descarte@hermetica.com>>.
 
+
 =head1 Additional DBI Information
 
 Additional information on the DBI project can be found on the World
